﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace PDT
{
    class DSKImage : DiskImage
    {
        public DSKImage(string filename)
            : base(filename)
        {         
        }

        public DSKImage(string filename, DiskContents contents): base (filename, contents)
        {             
        }

        public override void PrintAnalysis()
        {
            Utils.Multiprint("Analysis of non-protected disks is not (yet) supported",true);
        }

        public override void Write()
        {
            WriteDSK(_filename);
        }

        public override DiskContents Read()
        {            
            return null;
        }

        private void WriteDSK(string DSKName)
        {
            FileStream outputStream = new FileStream(DSKName, FileMode.Create);
            BinaryWriter writer = new BinaryWriter(outputStream);
            int numTracks = ((_contents.tracks.Count) / 40) * 40;
            int numSides = (_contents.tracks[0].sides[0].diskformat & 1) + 1;
            for (int t = 0; t < numTracks; t++)
            {
                for (int h = 0; h < numSides; h++)
                {
                    for (int s = 0; s < 9; s++)
                    {
                        WriteDSKSector(writer, t, h, s+1);
                    }
                }
            }
        }

        private void WriteDSKSector(BinaryWriter writer, int t, int h, int s)
        {
            int block = -1;
            int count = 0;
            foreach (SectorInfo sector in _contents.tracks[t].sides[h].sectors)
            {
                if ((sector.track == t) && (sector.head == h) && (sector.sector == s))
                {
                    block = count;
                }
                count++;
            }
            if (block == -1)    // not found
            {
                WriteBytes(writer, 0, 512);
            }
            else
            {
                byte[] contents = _contents.tracks[t].sides[h].sectors[block].contents;
                if (contents.Length >= 512)
                {
                    writer.Write(contents, 0, 512);
                }
                else
                {
                    writer.Write(contents, 0, contents.Length);
                    WriteBytes(writer, 0, (512 - contents.Length));
                }
            }
        }  
    }
}
